﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Ahp
{
    public class GoalNode : Node
    {
        /// <summary>
        /// Represents AHP goal node. This is root node of the hierarchy.
        /// </summary>
        public override decimal LocalPriority
        {
            get { return base.LocalPriority; }
            set { throw new InvalidOperationException("Changing local priority of the GoalNode is not allowed. Its value always equals 1."); }
        }

        public override decimal GlobalPriority
        {
            get { return 1M; }
        }

        private CriterionNodeCollection criterionNodes;
        /// <summary>
        /// Colletion of child criterion nodes
        /// </summary>
        public CriterionNodeCollection CriterionNodes
        {
            get { return criterionNodes; }
        }

        public GoalNode()
            : this(null)
        { }

        public GoalNode(string name)
            : base(name, 1M)
        {
            criterionNodes = new CriterionNodeCollection(HandleChildAdded, HandleChildRemoved);
        }

        /// <summary>
        /// Returns collection of criterion nodes, which are at lowest level in the hierarchy
        /// </summary>
        /// <returns></returns>
        public CriterionNodeCollection GetLowestCriterionNodes()
        {
            CriterionNodeCollection lowestCriterionNodes = new CriterionNodeCollection();

            foreach (CriterionNode criterionNode in criterionNodes)
            {
                LookForLowestCriterionNodes(criterionNode, lowestCriterionNodes);
            }

            return lowestCriterionNodes;
        }

        private void LookForLowestCriterionNodes(CriterionNode node, CriterionNodeCollection collection)
        {
            if (node.HasSubcriterionNodes)
            {
                foreach (CriterionNode subcriterionNode in node.SubcriterionNodes)
                {
                    LookForLowestCriterionNodes(subcriterionNode, collection);
                }
            }
            else
            {
                collection.Add(node);
            }
        }

        private void HandleChildAdded(CriterionNode node)
        {
            if (!Object.ReferenceEquals(node.GoalNode, this))
            {
                node.GoalNode = this;
            }
        }

        private void HandleChildRemoved(CriterionNode node)
        {
            if (Object.ReferenceEquals(node.GoalNode, this))
            {
                node.GoalNode = null;
            }
        }
    }
}
