﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using System.Xml;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Ahp;
using Ahp.Xml;
using System.IO;

namespace AhpTest
{
    [TestClass]
    public class HierarchyXmlSerializerTest
    {
        [TestMethod]
        public void SerializeTest()
        {
            //Иерархия            
            Hierarchy hierarchy = new Hierarchy("Choose optimal car");

            Alternative accordAlternative = hierarchy.Alternatives.Add("A", "Accord");
            Alternative accordHybridAlternative = hierarchy.Alternatives.Add("AH", "Accord Hybrid");
            Alternative civicAlternative = hierarchy.Alternatives.Add("C", "Civic");

            CriterionNode styleCriterion = hierarchy.GoalNode.CriterionNodes.Add("Style", 0.4M);
            CriterionNode capacityNode = hierarchy.GoalNode.CriterionNodes.Add("Capacity", 0.6M);

            CriterionNode cargoCapacityCriterion = capacityNode.SubcriterionNodes.Add("Cargo capacity", 0.2M);
            CriterionNode passengerCapacityCriterion = capacityNode.SubcriterionNodes.Add("Passenger capacity", 0.8M);

            hierarchy[accordAlternative, styleCriterion].LocalPriority = 0.350M;
            hierarchy[accordHybridAlternative, styleCriterion].LocalPriority = 0.350M;
            hierarchy[civicAlternative, styleCriterion].LocalPriority = 0.300M;

            hierarchy[accordAlternative, cargoCapacityCriterion].LocalPriority = 0.100M;
            hierarchy[accordHybridAlternative, cargoCapacityCriterion].LocalPriority = 0.100M;
            hierarchy[civicAlternative, cargoCapacityCriterion].LocalPriority = 0.800M;

            hierarchy[accordAlternative, passengerCapacityCriterion].LocalPriority = 0.150M;
            hierarchy[accordHybridAlternative, passengerCapacityCriterion].LocalPriority = 0.150M;
            hierarchy[civicAlternative, passengerCapacityCriterion].LocalPriority = 0.700M;

            //Ожидаемый XML
            StringBuilder stringBuilder = new StringBuilder();
            stringBuilder.Append("<?xml version=\"1.0\" encoding=\"utf-16\"?>");
            stringBuilder.Append("<hierarchy>");

            stringBuilder.Append("<alternatives>");
            stringBuilder.Append("<alternative>");
            stringBuilder.Append("<ID>A</ID>");
            stringBuilder.Append("<name>Accord</name>");
            stringBuilder.Append("</alternative>");
            stringBuilder.Append("<alternative>");
            stringBuilder.Append("<ID>AH</ID>");
            stringBuilder.Append("<name>Accord Hybrid</name>");
            stringBuilder.Append("</alternative>");
            stringBuilder.Append("<alternative>");
            stringBuilder.Append("<ID>C</ID>");
            stringBuilder.Append("<name>Civic</name>");
            stringBuilder.Append("</alternative>");
            stringBuilder.Append("</alternatives>");

            stringBuilder.Append("<goalNode>");
            stringBuilder.Append("<name>Choose optimal car</name>");
            stringBuilder.Append("<localPriority>1</localPriority>");

            stringBuilder.Append("<criterionNodes>");

            stringBuilder.Append("<criterionNode>");
            stringBuilder.Append("<name>Style</name>");
            stringBuilder.Append("<localPriority>0,4</localPriority>");
            stringBuilder.Append("<alternativeNodes>");
            stringBuilder.Append("<alternativeNode>");
            stringBuilder.Append("<alternativeID>A</alternativeID>");
            stringBuilder.Append("<localPriority>0,350</localPriority>");
            stringBuilder.Append("</alternativeNode>");
            stringBuilder.Append("<alternativeNode>");
            stringBuilder.Append("<alternativeID>AH</alternativeID>");
            stringBuilder.Append("<localPriority>0,350</localPriority>");
            stringBuilder.Append("</alternativeNode>");
            stringBuilder.Append("<alternativeNode>");
            stringBuilder.Append("<alternativeID>C</alternativeID>");
            stringBuilder.Append("<localPriority>0,300</localPriority>");
            stringBuilder.Append("</alternativeNode>");
            stringBuilder.Append("</alternativeNodes>");
            stringBuilder.Append("</criterionNode>");

            stringBuilder.Append("<criterionNode>");
            stringBuilder.Append("<name>Capacity</name>");
            stringBuilder.Append("<localPriority>0,6</localPriority>");
            stringBuilder.Append("<subcriterionNodes>");

            stringBuilder.Append("<criterionNode>");
            stringBuilder.Append("<name>Cargo capacity</name>");
            stringBuilder.Append("<localPriority>0,2</localPriority>");
            stringBuilder.Append("<alternativeNodes>");
            stringBuilder.Append("<alternativeNode>");
            stringBuilder.Append("<alternativeID>A</alternativeID>");
            stringBuilder.Append("<localPriority>0,100</localPriority>");
            stringBuilder.Append("</alternativeNode>");
            stringBuilder.Append("<alternativeNode>");
            stringBuilder.Append("<alternativeID>AH</alternativeID>");
            stringBuilder.Append("<localPriority>0,100</localPriority>");
            stringBuilder.Append("</alternativeNode>");
            stringBuilder.Append("<alternativeNode>");
            stringBuilder.Append("<alternativeID>C</alternativeID>");
            stringBuilder.Append("<localPriority>0,800</localPriority>");
            stringBuilder.Append("</alternativeNode>");
            stringBuilder.Append("</alternativeNodes>");
            stringBuilder.Append("</criterionNode>");

            stringBuilder.Append("<criterionNode>");
            stringBuilder.Append("<name>Passenger capacity</name>");
            stringBuilder.Append("<localPriority>0,8</localPriority>");
            stringBuilder.Append("<alternativeNodes>");
            stringBuilder.Append("<alternativeNode>");
            stringBuilder.Append("<alternativeID>A</alternativeID>");
            stringBuilder.Append("<localPriority>0,150</localPriority>");
            stringBuilder.Append("</alternativeNode>");
            stringBuilder.Append("<alternativeNode>");
            stringBuilder.Append("<alternativeID>AH</alternativeID>");
            stringBuilder.Append("<localPriority>0,150</localPriority>");
            stringBuilder.Append("</alternativeNode>");
            stringBuilder.Append("<alternativeNode>");
            stringBuilder.Append("<alternativeID>C</alternativeID>");
            stringBuilder.Append("<localPriority>0,700</localPriority>");
            stringBuilder.Append("</alternativeNode>");
            stringBuilder.Append("</alternativeNodes>");
            stringBuilder.Append("</criterionNode>");
            stringBuilder.Append("</subcriterionNodes>");

            stringBuilder.Append("</criterionNode>");
            stringBuilder.Append("</criterionNodes>");

            stringBuilder.Append("</goalNode>");
            stringBuilder.Append("</hierarchy>");

            //Проверка
            MemoryStream stream = new MemoryStream();
            HierarchyXmlSerializer serializer = new HierarchyXmlSerializer();
            string expectedXml = stringBuilder.ToString();
            string realXml = serializer.Serialize(hierarchy);

            Assert.AreEqual(expectedXml, realXml);
        }
    }
}
